@@ -117,32 +117,6 @@ class ParallelOptimizationTest(unittest.TestCase):
 
         return jacobian
 
-    def test_gna_emg_conditions(self):
-
-        for p in [self.batch_initials.clone().float(), self.batch_initials.clone().double()]:
-
-            coeffs = lsq_gna_parallel(
-                p = p,
-                function = self.multi_cost_batch,
-                jac_function = self.multi_jaco_batch,
-                args = (self.t.to(dtype=p.dtype), self.batch_data_channels.to(dtype=p.dtype)),
-                wvec = torch.ones(2, device=self.device, dtype=p.dtype, requires_grad=False),
-                l = 1,
-                ftol = 1e-8,
-                ptol = 1e-8,
-                gtol = 1e-8,
-                max_iter = 99,
-            )
-
-            # assertion
-            self.iter_gna_cond = len(coeffs)
-            self.assertTrue(self.iter_gna_cond < 100, 'Number of iterations exceeded 100')
-            ret_params = torch.allclose(coeffs[-1].double(), self.gt_params, atol=1e-1)
-            self.assertTrue(ret_params, 'Coefficients deviate')
-            eps = torch.sum(self.cost_batch(coeffs[-1].double(), t=self.t, y=self.batch_data_channels)).cpu()
-            self.eps_gna_cond = torch.round(eps, decimals=4)
-            self.assertTrue(self.eps_gna_cond/len(self.gt_params) < 1, 'Error exceeded 1')
-
     def test_gna_emg_plain(self):
 
         for p in [self.batch_initials.clone().float(), self.batch_initials.clone().double()]:
@@ -167,6 +141,91 @@ class ParallelOptimizationTest(unittest.TestCase):
             self.eps_gna = torch.round(eps, decimals=4)
             self.assertTrue(self.eps_gna/len(self.gt_params) < 1, 'Error exceeded 1')
 
+    def test_gda_emg_plain(self):
+
+        from torchimize.functions.parallel.gda_fun_parallel import gradient_descent_parallel_plain
+
+        for p in [self.batch_initials.clone().float(), self.batch_initials.clone().double()]:
+
+            multi_cost_batch_args = lambda p: self.multi_cost_batch(p_batch=p, t=self.t.to(dtype=p.dtype), y=self.batch_data_channels.to(dtype=p.dtype))
+            multi_jaco_batch_args = lambda p: self.multi_jaco_batch(p_batch=p, t=self.t.to(dtype=p.dtype), data=self.batch_data_channels.to(dtype=p.dtype))
+
+            coeffs = gradient_descent_parallel_plain(
+                p = p,
+                function = multi_cost_batch_args,
+                jac_function = multi_jaco_batch_args,
+                wvec = torch.ones(2, device=self.device, dtype=p.dtype, requires_grad=False),
+                l = 1,
+                max_iter = 99,
+            )
+
+            # assertion
+            ret_params = torch.allclose(coeffs.double(), self.gt_params, atol=1e-1)
+            self.assertTrue(ret_params, 'Coefficients deviate')
+            eps = torch.sum(multi_cost_batch_args(coeffs.double())).cpu()
+            self.eps_gna = torch.round(eps, decimals=4)
+            self.assertTrue(self.eps_gna/len(self.gt_params) < 1, 'Error exceeded 1')
+
+    def test_gradient_descent_emg_conditions(self):
+
+        from torchimize.functions.parallel.gda_fun_parallel import gradient_descent_parallel
+
+        for p in [self.batch_initials.clone().float(), self.batch_initials.clone().double()]:
+
+            # test numerical and analytical jacobian
+            for jaco in [None, self.multi_jaco_batch]:
+
+                coeffs = gradient_descent_parallel(
+                    p = p,
+                    function = self.multi_cost_batch,
+                    jac_function = jaco,
+                    args = (self.t.to(dtype=p.dtype), self.batch_data_channels.to(dtype=p.dtype)),
+                    wvec = torch.ones(2, device=self.device, dtype=p.dtype, requires_grad=False),
+                    l = 1,
+                    ftol = 1e-5 if jaco is None else 1e-8,
+                    ptol = 1e-5 if jaco is None else 1e-8,
+                    gtol = 1e-5 if jaco is None else 1e-8,
+                    max_iter = 99,
+                )
+
+                # assertion
+                self.iter_gd_cond = len(coeffs)
+                self.assertTrue(self.iter_gd_cond < 100, 'Number of iterations exceeded 100')
+                ret_params = torch.allclose(coeffs[-1].double(), self.gt_params, atol=1e-1)
+                self.assertTrue(ret_params, 'Coefficients deviate')
+                eps = torch.sum(self.cost_batch(coeffs[-1].double(), t=self.t, y=self.batch_data_channels)).cpu()
+                self.eps_gd_cond = torch.round(eps, decimals=4)
+                self.assertTrue(self.eps_gd_cond/len(self.gt_params) < 1, 'Error exceeded 1')
+
+    def test_gna_emg_conditions(self):
+
+        for p in [self.batch_initials.clone().float(), self.batch_initials.clone().double()]:
+
+            # test numerical and analytical jacobian
+            for jaco in [None, self.multi_jaco_batch]:
+
+                coeffs = lsq_gna_parallel(
+                    p = p,
+                    function = self.multi_cost_batch,
+                    jac_function = jaco,
+                    args = (self.t.to(dtype=p.dtype), self.batch_data_channels.to(dtype=p.dtype)),
+                    wvec = torch.ones(2, device=self.device, dtype=p.dtype, requires_grad=False),
+                    l = 1,
+                    ftol = 1e-5 if jaco is None else 1e-8,
+                    ptol = 1e-5 if jaco is None else 1e-8,
+                    gtol = 1e-5 if jaco is None else 1e-8,
+                    max_iter = 99,
+                )
+
+                # assertion
+                self.iter_gna_cond = len(coeffs)
+                self.assertTrue(self.iter_gna_cond < 100, 'Number of iterations exceeded 100')
+                ret_params = torch.allclose(coeffs[-1].double(), self.gt_params, atol=1e-1)
+                self.assertTrue(ret_params, 'Coefficients deviate')
+                eps = torch.sum(self.cost_batch(coeffs[-1].double(), t=self.t, y=self.batch_data_channels)).cpu()
+                self.eps_gna_cond = torch.round(eps, decimals=4)
+                self.assertTrue(self.eps_gna_cond/len(self.gt_params) < 1, 'Error exceeded 1')
+
     def test_lma_emg_conditions(self):
 
         from torchimize.functions.parallel.lma_fun_parallel import lsq_lma_parallel
@@ -174,28 +233,31 @@ class ParallelOptimizationTest(unittest.TestCase):
         for p in [self.batch_initials.clone().float(), self.batch_initials.clone().double()]:
             
             for m in ['lev']:
+                
+                # test numerical and analytical jacobian
+                for jaco in [None, self.multi_jaco_batch]:
 
-                coeffs = lsq_lma_parallel(
-                    p = p,
-                    function = self.multi_cost_batch,
-                    jac_function = self.multi_jaco_batch,
-                    args = (self.t.to(dtype=p.dtype), self.batch_data_channels.to(dtype=p.dtype)),
-                    wvec = torch.ones(2, device=self.device, dtype=p.dtype, requires_grad=False),
-                    meth = m,
-                    ftol = 1e-8,
-                    ptol = 1e-8,
-                    gtol = 1e-8,
-                    max_iter = 99,
-                )
+                    coeffs = lsq_lma_parallel(
+                        p = p,
+                        function = self.multi_cost_batch,
+                        jac_function = jaco,
+                        args = (self.t.to(dtype=p.dtype), self.batch_data_channels.to(dtype=p.dtype)),
+                        wvec = torch.ones(2, device=self.device, dtype=p.dtype, requires_grad=False),
+                        meth = m,
+                        ftol = 1e-5 if jaco is None else 1e-8,
+                        ptol = 1e-5 if jaco is None else 1e-8,
+                        gtol = 1e-5 if jaco is None else 1e-8,
+                        max_iter = 99,
+                    )
 
-                # assertion
-                self.iter_lma_cond = len(coeffs)
-                self.assertTrue(self.iter_lma_cond < 100, 'Number of iterations exceeded 100')
-                ret_params = torch.allclose(coeffs[-1].double(), self.gt_params, atol=1e-1)
-                self.assertTrue(ret_params, 'Coefficients deviate')
-                eps = torch.sum(self.cost_batch(coeffs[-1].double(), t=self.t, y=self.batch_data_channels)).cpu()
-                self.eps_lma_cond = torch.round(eps, decimals=4)
-                self.assertTrue(self.eps_lma_cond/len(self.gt_params) < 1, 'Error exceeded 1')
+                    # assertion
+                    self.iter_lma_cond = len(coeffs)
+                    self.assertTrue(self.iter_lma_cond < 100, 'Number of iterations exceeded 100')
+                    ret_params = torch.allclose(coeffs[-1].double(), self.gt_params, atol=1e-1)
+                    self.assertTrue(ret_params, 'Coefficients deviate')
+                    eps = torch.sum(self.cost_batch(coeffs[-1].double(), t=self.t, y=self.batch_data_channels)).cpu()
+                    self.eps_lma_cond = torch.round(eps, decimals=4)
+                    self.assertTrue(self.eps_lma_cond/len(self.gt_params) < 1, 'Error exceeded 1')
 
     def scipy_fit(self):
 
@@ -238,6 +300,8 @@ class ParallelOptimizationTest(unittest.TestCase):
 
         self.test_fun_dims()
         self.test_gna_emg_plain()
+        self.test_gda_emg_plain()
+        self.test_gradient_descent_emg_conditions()
         self.test_gna_emg_conditions()
         self.test_lma_emg_conditions()
 
