@@ -2,9 +2,17 @@ import torch
 from torch import Tensor
 import torch.nn as nn
 from torch.optim.optimizer import Optimizer
-from torch.nn.utils import _stateless
 from typing import List
 
+from distutils.version import LooseVersion
+if LooseVersion(torch.__version__) >= LooseVersion('2.0.0'):
+    from torch.func import functional_call
+else:
+    try:
+        from torch.nn.utils.stateless import functional_call
+    except ImportError:
+        from torch.nn.utils._stateless import functional_call
+
 
 class GNA(Optimizer):
     r"""Implements Gauss-Newton.
@@ -63,7 +71,7 @@ class GNA(Optimizer):
             if self.hessian_approx:
                 # vectorized jacobian (https://github.com/pytorch/pytorch/issues/49171)
                 def func(*params: torch.Tensor):
-                    out = _stateless.functional_call(self._model, {n: p for n, p in zip(keys, params)}, x)
+                    out = functional_call(self._model, {n: p for n, p in zip(keys, params)}, x)
                     return out
                 self._j_list: tuple[torch.Tensor] = torch.autograd.functional.jacobian(func, values, create_graph=False)    # NxCxBxCxHxW
                 # create hessian approximation
@@ -78,7 +86,7 @@ class GNA(Optimizer):
             else:
                 # vectorized hessian (https://github.com/pytorch/pytorch/issues/49171)
                 def func(*params: torch.Tensor):
-                    out: torch.Tensor = _stateless.functional_call(self._model, {n: p for n, p in zip(keys, params)}, x)
+                    out: torch.Tensor = functional_call(self._model, {n: p for n, p in zip(keys, params)}, x)
                     return out.square().sum()
                 self._h_list: tuple[torch.Tensor] = torch.autograd.functional.hessian(func, tuple(self._model.parameters()), create_graph=False)
                 self._h_list = [self._h_list[i][i] for i in range(len(self._h_list))] # filter j-th element
@@ -121,4 +129,4 @@ class GNA(Optimizer):
                 d2_p = h_i.matmul(d_p.flatten()).reshape(d_p_list[i].shape)
                 param.add_(d2_p, alpha=-lr)
             else:
-                raise Exception('Tensor dimension mismatch')
\ No newline at end of file
+                raise Exception('Tensor dimension mismatch')
